SystemOrganization addCategory: #HistoryCollection!

Object subclass: #HistoryCollection
	instanceVariableNames: 'storage maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'HistoryCollection'!

!HistoryCollection commentStamp: 'BenjaminVanRyseghem 3/17/2011 13:41' prior: 0!
I'm a data structure used to store an history.

My behavior is
	- I store Associations (anElement -> anInteger), where anElement is the element to store and anInteger the number of times it has been added. (as a Dictionary)
	- I only store once each element (as a Set).
	- I have a max size. If I add an element and have reached the max size, I remove the less seen element.
	- I store element in the chronological order (as an OrderedCollection)!

!HistoryCollection class methodsFor: 'instance creation' stamp: 'BenjaminVanRyseghem 3/17/2011 13:47'!
maxSize: anInteger

	^ self new maxSize: anInteger! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:37'!
add: anElement

	| count |
	count := 0.
	(self includes: anElement)
		ifTrue: [ | association |
			association := self remove: anElement.
			count := association value ].
	self size = self maxSize
		ifTrue: [	self removeLast ].

	^ storage addFirst: (anElement -> (count+1))! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:43'!
at: anElement

	^ storage detect: [:each | each key = anElement]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:38'!
identityIncludes: anElement

	^ storage anySatisfy: [:association | association key == anElement ]! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:37'!
includes: anElement

	^ storage anySatisfy: [:association | association key = anElement ]! !

!HistoryCollection methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/17/2011 13:47'!
initialize

	super initialize.
	maxSize := 5.
	storage := OrderedCollection new: maxSize.! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:00'!
lessSeen

	| min |
	min := storage first.
	2 to: storage size do: [:index || each |
		each := storage at: index.
		(each value < min value)
			ifTrue: [ min := each]].
	^ min key! !

!HistoryCollection methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/17/2011 13:31'!
maxSize

	^ maxSize! !

!HistoryCollection methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 3/17/2011 13:32'!
maxSize: anInteger

	maxSize := anInteger! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:05'!
mostSeen

	| max |
	max := storage first.
	2 to: storage size do: [:index || each |
		each := storage at: index.
		(each value > max value)
			ifTrue: [ max := each]].
	^ max key! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:03'!
oldest

	^ storage last key! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:49'!
remove: anElement

	| association |
	association := self at: anElement.
	^ storage remove: association.
	! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:54'!
removeLast
	" I remove the less seen one, in case of equality, I remove the latest one "


	| min reversedStorage |
	reversedStorage := storage reverse.
	min := reversedStorage first.
	2 to: storage size do: [:index || each |
		each := reversedStorage at: index.
		(each value < min value)
			ifTrue: [ min := each ]].
	storage remove: min! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 13:49'!
size

	^ storage size! !

!HistoryCollection methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 3/17/2011 14:03'!
youngest

	^ storage first key! !
